from fastapi import Request, HTTPException, Depends
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials

from .auth import decodeJWT
from sredis.models import PUser


class JWTBearer(HTTPBearer):
    def __init__(self, auto_error: bool = True):
        super(JWTBearer, self).__init__(auto_error=auto_error)

    async def __call__(self, request: Request):
        credentials: HTTPAuthorizationCredentials = await super(
            JWTBearer, self
        ).__call__(request)
        if credentials:
            if not credentials.scheme == "Bearer":
                raise HTTPException(
                    status_code=403, detail="Invalid authentication scheme."
                )
            cred = self.verify_jwt(credentials.credentials)
            if not cred:
                raise HTTPException(
                    status_code=403, detail="Invalid token or expired token."
                )
            return credentials.credentials
        else:
            raise HTTPException(status_code=403, detail="Invalid authorization code.")

    def verify_jwt(self, jwtoken: str) -> bool:
        isTokenValid: bool = False

        payload = decodeJWT(jwtoken)
        if payload:
            return payload
        return isTokenValid


async def get_current_user(token: str = Depends(JWTBearer())) -> str:
    payload = decodeJWT(token)
    return payload["user_id"]


async def admin_user(token: str = Depends(JWTBearer())) -> str:
    payload = decodeJWT(token)
    user = PUser.find(PUser.username == payload["user_id"]).all()[0]
    if not user.admin:
        raise HTTPException(status_code=401, detail="you can not have access")
    return payload["user_id"]
