from fastapi import FastAPI, Response, Body, Depends
from typing import List
from balancer.util import proxy_datacenters
from sredis.sredis import check_user, create_puser, add_datacenter, set_token
import logging
import requests
from core.models import User, DataCenter, Token, UserLoginSchema
from core.auth import signJWT
from core.bearer import get_current_user, admin_user
from redis_om import Migrator

logging.config.fileConfig("logging.conf", disable_existing_loggers=False)

# get root logger
logger = logging.getLogger(__name__)
requests.packages.urllib3.disable_warnings()
Migrator().run()

app = FastAPI()
add_datacenter("https://kappa1.fured.cloud.bme.hu")
add_datacenter("https://kappa2.fured.cloud.bme.hu")
add_datacenter("https://kappa3.fured.cloud.bme.hu")


@app.post("/user/signup", tags=["user"])
async def create_user(user: User = Body(...)):
    create_puser(user)
    return signJWT(user.username)


@app.post("/user/login", tags=["user"])
async def user_login(user: UserLoginSchema = Body(...)):
    if check_user(user):
        return signJWT(user.username)
    return {"error": "Wrong login details!"}


@app.get("/lb/{server_path:path}")
def proxy_get(server_path: str = "/", username=Depends(get_current_user)):
    return proxy_datacenters(server_path, username)


@app.post("/lb/{server_path:path}")
def proxy_post(server_path: str = "/", username=Depends(get_current_user)):
    return proxy_datacenters(server_path, username, method="POST")


@app.post("/add_datacenter/")
def create_datacenter(dc: DataCenter = None, username=Depends(admin_user)):
    add_datacenter(dc.name)
    return Response(status_code=201)


@app.post("/set_tokens/")
def set_tokens(tokens: List[Token] = None, username=Depends(get_current_user)):
    for token in tokens:
        set_token(username, str(token.datacenter), str(token.token))
    return tokens
