import redis
from core.models import UserLoginSchema
from .models import PUser
from passlib.hash import pbkdf2_sha256
from core.models import User
from core.auth import hash_pass
from fastapi import HTTPException

r = redis.Redis(host="localhost", port=6379, db=0, decode_responses=True)
r.set("datacenters_cnt", 1)
all_keys = list(r.hgetall("datacenters_hash").keys())
if all_keys:
    r.hdel("datacenters_hash", *all_keys)


def add_datacenter(datacenter: str):
    cnt = int(r.get("datacenters_cnt"))
    r.hset("datacenters_hash", cnt, datacenter)
    r.incr("datacenters_cnt")
    r.set("roundrobin_cnt", 1)


def rr_get():
    cnt = int(r.get("datacenters_cnt"))
    rr = int(r.get("roundrobin_cnt"))
    if rr + 1 >= cnt:
        r.set("roundrobin_cnt", 1)
    else:
        r.incr("roundrobin_cnt")
    return str(r.hget("datacenters_hash", rr))


def set_token(username: str, datacenter: str, token: str):
    print(f"tokens:{username}" + datacenter)
    r.hset(f"tokens:{username}", datacenter, token)


def get_datacenter_token(username: str, datacenter: str):
    return str(r.hget(f"tokens:{username}", datacenter))


def check_user(data: UserLoginSchema):
    user = PUser.find(PUser.username == data.username).all()
    if pbkdf2_sha256.verify(data.password, user[0].password):
        return user[0]
    return False


def create_puser(user: User):
    s = PUser.find(PUser.username == user.username).all()
    if s:
        raise HTTPException(status_code=403, detail="User already exists")
    user = PUser(
        username=user.username, email=user.email, password=hash_pass(user.password)
    )
    user.save()
